#include "../config.h"
#include <sys/types.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

/* Write to a file, handling transient errors. */
ssize_t
retry_write(int fd, unsigned char *buf, size_t length)
{
	size_t start;
	ssize_t i;
	start = 0;
	while (start < length) {
		i = write(fd, buf + start, length - start);
		switch (i) {
		case 0:
			return start;
			break;
		case -1:
			switch (errno) {
			case EAGAIN:
				continue;
			default:
				return -1;
			}
			break;
		default:
			start += i;
			break;
		}
	}
	return start;
}

void *
oddjob_malloc(size_t size)
{
	void *ret;
	ret = malloc(size);
	if (ret == NULL) {
		fprintf(stderr, "Out of memory!\n");
		_exit(1);
	}
	return ret;
}

void *
oddjob_malloc0(size_t size)
{
	void *ret;
	ret = oddjob_malloc(size);
	memset(ret, 0, size);
	return ret;
}

void
oddjob_free(void *p)
{
	free(p);
}

char *
oddjob_strdup(const char *s)
{
	char *r;
	r = oddjob_malloc0(strlen(s) + 1);
	strcpy(r, s);
	return r;
}

char *
oddjob_strndup(const char *s, int n)
{
	char *r;
	int len;
	len = strlen(s);
	if (len > n) {
		len = n;
	}
	r = oddjob_malloc0(len + 1);
	memmove(r, s, len);
	return r;
}

void
oddjob_resize_array(void **array, size_t element_size,
		    size_t current_n_elements, size_t want_n_elements)
{
	void *p;
	if (want_n_elements > 0xffff) {
		fprintf(stderr, "Internal limit exceeded.\n");
		_exit(1);
	}
	if (element_size > 0xffff) {
		fprintf(stderr, "Internal limit exceeded.\n");
		_exit(1);
	}
	p = oddjob_malloc0(element_size * want_n_elements);
	if (current_n_elements > 0) {
		memmove(p, *array, element_size * current_n_elements);
		memset(*array, 0, element_size * current_n_elements);
	}
	oddjob_free(*array);
	*array = p;
}
